rem Author:  Mark Lang, 1998
rem Name:    crrbs.sql
rem Purpose: Create initial public rollback segments
rem Usage:   @crrbs <num> <ext_size_k> <ext_min/opt> <ts>
rem Subject: database
rem Attrib:  sql gen ddl dba
rem Descr:
rem Notes:   Example: Create 4 public rollbacks with 10 100k extents
rem          	@crrbs 4 100 10 rbs
rem SeeAlso:
rem          01-feb-98  Initial release

@setup
set heading off

define num="'&&1'"
define siz="'&&2'"
define min="'&&3'"
define ts ="upper('&&4')"

spool crrbs.tmp

select
  'rem *** Create Startup RBS R0 ***'||&&cr||
  'CREATE PUBLIC ROLLBACK SEGMENT r0 TABLESPACE system;'||&&cr||
  'ALTER ROLLBACK SEGMENT r0 ONLINE;'
from dual;

select
  'rem *** Set existing rollbacks offline ***'
from dual;

select
  'ALTER ROLLBACK SEGMENT '||segment_name||' OFFLINE;'
from sys.dba_rollback_segs
where segment_name NOT IN ('SYSTEM', 'R0')
and tablespace_name = &&ts
order by segment_name;

select
  'rem *** Drop existing rollbacks ***'
from dual;

select
  'DROP ROLLBACK SEGMENT '||segment_name||';'
from sys.dba_rollback_segs
where segment_name NOT IN ('SYSTEM', 'R0')
and tablespace_name = &&ts
order by segment_name;

select
  'rem *** Create new rollbacks with user parameters ***'
from dual;

select
  'CREATE PUBLIC ROLLBACK SEGMENT r'||ltrim(to_char(rownum,'00'))||CHR(10)||
  'TABLESPACE '||&&ts||CHR(10)||
  'STORAGE (INITIAL '||&&siz||'k NEXT '||&&siz||'k MINEXTENTS '||&&min||
    ' OPTIMAL '||&&siz*&&min||'k);'
from sys.obj$
where rownum <= &&1;

select
  'rem *** Set rollbacks online ***'
from dual;

select
  'ALTER ROLLBACK SEGMENT r'||ltrim(to_char(rownum,'00'))||' ONLINE;'
from sys.obj$
where rownum <= &&1;

select
  'rem *** Set R0 offline ***'||&&cr||
  'ALTER ROLLBACK SEGMENT r0 OFFLINE;'
from dual;

select
  'rem *** Display rollback status ***'||&&cr||
  'SET HEADING ON'||&&cr||
  'select segment_name, status from sys.dba_rollback_segs;'
from dual;

spool off
@doit
set feedback on
@crrbs.tmp

undef num siz min ts

@setdefs

