rem Author:  Mark Lang, 1998
rem Name:    crseqs.sql
rem Purpose: Rebuild sequences which follow specific naming standards
rem Usage:   @crseqs <%owner.sequence%>
rem Attrib:  sql
rem Descr:
rem Notes:   Following naming standards must be observed to use this
rem          script:
rem            table PK = <table_short_name>_PK
rem            sequence = <table_short_name>_<seq_column>_SEQ
rem SeeAlso: @crseq @seqs
rem History:
rem          05-feb-98  Initial release

@setup1
set serveroutput on size 102400

spool crseqs.tmp

declare
  w1 varchar2(30);
  w2 varchar2(30);
  w3 varchar2(30);
  n1 number;
  t1 varchar2(30);
  c1 varchar2(30);
  d1 varchar2(30);
begin
  for r in (
    select *
    from all_sequences
    where sequence_owner like &&o1
    and sequence_name like &&n1
  ) loop 
    n1:=instr(r.sequence_name,'_');
    w1:=substr(r.sequence_name,1,n1-1);
    w2:=substr(r.sequence_name,n1+1,length(r.sequence_name)-n1-4);
    w3:=substr(r.sequence_name,length(r.sequence_name)-2);
    --&&putl(w1||','||w2||','||w3);
    if w3='SEQ' then
      begin
        select c.table_name,c.column_name,l.data_type
        into t1,c1,d1
        from all_cons_columns c, all_tab_columns l
        where c.owner=r.sequence_owner
        and c.constraint_name=w1||'_'||'PK'
        and c.owner=l.owner
        and c.table_name=l.table_name
        and c.column_name=l.column_name;
        if d1<>'NUMBER' then
          &&putl('rem '||r.sequence_name||' refs '||t1||'.'||c1||
            ' which is not a NUMBER');
        else
          &&putl('@crseq '||r.sequence_name||' '||t1||' '||c1);
        end if;
      exception
        when no_data_found then
          &&putl('rem Do not know how to recreate '||r.sequence_name);
        when too_many_rows then
          &&putl('rem Finding too many columns for '||r.sequence_name);
      end;
    end if;  
  end loop;
end;
/

spool off
@doit 
set feedback on
@crseqs.tmp
@setdefs

