rem Author:  Mark Lang, 1998
rem Name:    dview.sql
rem Purpose: Dump view definition(s) to a flat file
rem Usage:   @dview <%owner.view%> <filename>
rem Subject: object:view
rem Attrib:  sql
rem Descr:
rem Notes:
rem SeeAlso:
rem History:

@setup1
set heading off
set linesize 70
set serveroutput on size 500000

SELECT
  'Dumping '||count(*)||' view(s) '||
  DECODE(SUM(DECODE(o.status, 'VALID', 0, 1)), 0, '', NULL, '',
    '('||SUM(DECODE(o.status, 'VALID', 0, 1))||' invalid) ')||
  'like "'||UPPER('&&1')||'" to '||
  '"'||UPPER('&&2')||'"...'
FROM sys.dba_views v, sys.dba_objects o
WHERE v.owner LIKE &&o1 AND v.view_name LIKE &&n1
AND v.owner = o.owner AND v.view_name = o.object_name
AND o.object_type = 'VIEW'
;

SPOOL &&2

DECLARE
  n_i NUMBER;
  n_j NUMBER;
  n_k NUMBER;
  n_l NUMBER;
  n_m NUMBER;
  n_n NUMBER;
  n_z NUMBER;
  l_c LONG;
  v_t VARCHAR2(80);

  PROCEDURE pl (s VARCHAR2)
  IS 
    x VARCHAR2(255);
  BEGIN
    x := REPLACE(RTRIM(s), CHR(0));
    IF NVL(LENGTH(x),0) > 0 THEN
      &&putl(x);
    END IF;
  END;

BEGIN
  n_z := 1;
  FOR r IN (
    SELECT
      owner, view_name
    FROM sys.dba_views
    WHERE owner LIKE &&o1
    AND view_name LIKE &&n1
    ORDER BY owner, view_name
  ) LOOP

    /* comment */
    SELECT
      DECODE(o.status, 'VALID', null, '/* this view is invalid as of '||
      TO_CHAR(SYSDATE, 'DD-MON-YY HH:MI:SS')||' */')
    INTO v_t
    FROM sys.dba_objects o
    WHERE o.owner = r.owner
    AND o.object_name = r.view_name
    AND o.object_type = 'VIEW';

    IF v_t IS NOT NULL THEN
      pl(v_t);
      n_z := n_z + 1;
    END IF;

    /* create view command */
    pl('CREATE OR REPLACE FORCE VIEW '||r.owner||'.'||r.view_name);
    n_z := n_z + 1;

    /* list column headers */
    v_t := '(';
    SELECT
      MAX(column_id) INTO n_n
    FROM sys.dba_tab_columns
    WHERE owner = r.owner
    AND table_name = r.view_name;

    FOR s IN (
      SELECT
        column_name, column_id
      FROM all_tab_columns
      WHERE owner = r.owner
      AND table_name = r.view_name
      ORDER BY column_id
    ) LOOP

      IF LENGTH(v_t) + LENGTH(s.column_name) > 79-3 THEN
        pl(v_t);
        v_t := '';
        n_z := n_z + 1;
      END IF;

      v_t := v_t||'"'||s.column_name||'"';

      IF s.column_id < n_n THEN
        v_t := v_t||',';
      ELSE
        v_t := v_t||')';
      END IF;
    END LOOP;

    pl(v_t);
    n_z := n_z + 1;

    pl('AS');
    n_z := n_z + 1;

    SELECT
      text INTO l_c
    FROM sys.dba_views
    WHERE owner = r.owner
    AND view_name = r.view_name;

    n_i := 1;

    WHILE (n_i < LENGTH(l_c)) LOOP
      n_j := INSTR(l_c, CHR(10), n_i, 1) - n_i + 1;

      IF n_j > 0 AND n_j < 71 THEN
        v_t := SUBSTR(l_c, n_i, n_j);

        pl(REPLACE(SUBSTR(v_t, 1, n_j-1), CHR(9), ' '));

        n_i := n_i + n_j;
        n_z := n_z + 1;

      ELSE

        n_j := LEAST(LENGTH(l_c) - n_i + 1, 79);
        v_t := SUBSTR(l_c, n_i, n_j);
        n_k := 0;

        WHILE INSTR(v_t, CHR(39), 1, n_k+1) > 0 LOOP
          n_k := n_k + 1;
        END LOOP;

        IF MOD(n_k, 2) = 1 THEN
          /* unfinished ' string */
          n_l := INSTR(v_t, CHR(39), 1, n_k);

          pl(REPLACE(SUBSTR(v_t, 1, n_l-1), CHR(9), ' '));

          n_i := n_i + n_l-1;
          n_z := n_z + 1;

        ELSE
          n_m := 0;

          WHILE INSTR(v_t, '"', GREATEST(1, n_k), n_m+1) > 0 LOOP
            n_m := n_m + 1;
          END LOOP;

          IF MOD(n_m, 2) = 1 THEN
            /* unfinished " string */
            n_n := INSTR(v_t, '"', 1, n_m);

            pl(REPLACE(SUBSTR(v_t, 1, n_n-1), CHR(9), ' '));

            n_i := n_i + n_n-1;
            n_z := n_z + 1;

          ELSE
            /* wrap text */
            n_n := GREATEST(
                            INSTR(v_t, CHR(39), -1, 1),
                            INSTR(v_t, '"', -1, 1),
                            INSTR(v_t, ' ', -1, 1),
                            INSTR(v_t, ',', -1, 1),
                            INSTR(v_t, '|', -1, 1)
                           );

            IF n_n > 0 THEN

              pl(REPLACE(SUBSTR(v_t, 1, n_n), CHR(9), ' '));

              n_i := n_i + n_n;

            ELSE
              pl(REPLACE(v_t, CHR(9), ' '));

              n_i := n_i + LENGTH(v_t);

            END IF;

            n_z := n_z + 1;

          END IF;
        END IF;
      END IF;

    END LOOP;

    pl(';');
    n_z := n_z + 1;
    pl(' ');
    n_z := n_z + 1;
    COMMIT;
  END LOOP;
END;
/

SPOOL OFF

@setdefs

