rem Author:  Mark Lang, 1998
rem Name:    events.sql
rem Purpose: Display system events from V$SYSTEM_EVENT
rem Usage:   @events <%user%|avg|0> <%search%> <min_wait>
rem Subject: tuning
rem Attrib:  sql dab
rem Descr:
rem Notes:
rem SeeAlso: @waits
rem History:
rem          01-feb-98  Initial release

@setup
define db=""
define usr="upper('&&1')"
define stat="upper('&&2')"
define min="to_number(replace('&&3','%','0'))"

column username format a12
column event format a30
column total_waits format 999990 heading "WAITS"
column total_timeouts format 9990 heading "TOUTS"
column time_waited format 999999990 heading "TIME_WAITED"
column average_wait format 99990.99 heading "AVG_WAIT"

select
  s.username,
  e.event,
  e.total_waits,
  e.total_timeouts,
  e.time_waited,
  e.average_wait
from v$session_event&&db e, v$session&&db s
where &&usr not in ('0','AVG','SUM','MIN','MAX','STD','VAR')
and s.username like &&usr
and e.sid=s.sid
and upper(e.event) like &&stat
and e.time_waited>&&min
order by time_waited
;

select
e.event
,decode(&&usr
  ,'AVG', avg(e.total_waits)
  ,'SUM', sum(e.total_waits)
  ,'MIN', min(e.total_waits)
  ,'MAX', max(e.total_waits)
  ,'STD', stddev(e.total_waits)
  ,'VAR', variance(e.total_waits)
) total_waits
,decode(&&usr
  ,'AVG', avg(e.total_timeouts)
  ,'SUM', sum(e.total_timeouts)
  ,'MIN', min(e.total_timeouts)
  ,'MAX', max(e.total_timeouts)
  ,'STD', stddev(e.total_timeouts)
  ,'VAR', variance(e.total_timeouts)
) total_timeouts
,decode(&&usr
  ,'AVG', avg(e.time_waited)
  ,'SUM', sum(e.time_waited)
  ,'MIN', min(e.time_waited)
  ,'MAX', max(e.time_waited)
  ,'STD', stddev(e.time_waited)
  ,'VAR', variance(e.time_waited)
) time_waited
,decode(&&usr
  ,'AVG', avg(e.average_wait)
  ,'SUM', sum(e.average_wait)
  ,'MIN', min(e.average_wait)
  ,'MAX', max(e.average_wait)
  ,'STD', stddev(e.average_wait)
  ,'VAR', variance(e.average_wait)
) average_wait
from v$session_event&&db e
where &&usr in ('AVG','SUM','MIN','MAX','STD','VAR')
and upper(e.event) like &&stat
and e.time_waited>&&min
group by e.event
order by 4
;


select
  event,
  total_waits,
  total_timeouts,
  time_waited,
  average_wait
from v$system_event&&db
where &&usr='0'
and upper(event) like &&stat
order by time_waited
;

undef db usr stat min

@setdefs


