rem Author:  Mark Lang, 1998
rem Name:    hier.sql
rem Purpose: Display role hierarchy for a parent role
rem Usage:   @hier <role> <levels|%>
rem Subject: security
rem Attrib:  pls dba n80
rem Descr:
rem Notes:   if levels>0 then show to N level
rem          if levels=0 then show unlimited depth
rem          if levels<0 then show parents
rem          % defaults to "0"
rem          enh: Cant use in v8 because TYPE is TYPE# in SYS.USER$
rem SeeAlso: @privs
rem History:
rem          05-feb-98  Add support for version 8
rem          01-feb-98  Initial release
rem          19-jan-98  Indicate if role is password protected


@setup
@setver
set serveroutput on size 102400
set heading off

define usr="upper('&&1')"
define lvl="&&2"
define lv2="to_number(replace('&&2','%','0'))"

declare
  n number;
  v_text varchar2(80);
  v_name varchar2(30);
  n_type number;
  v_out  varchar2(240);

  function get_role_name(i number)
  return varchar2 is
    retval varchar2(30);
  begin
    select name||decode(password, null, '', ' (P)')
    into retval
    from sys.user$
    where user#=i;
    return retval;
  end;

  function get_def_role(u number, r number)
  return number is
    n_defrole number;
    n_found number;
  begin
    select defrole into n_defrole
    from sys.user$
    where user#=u;
    if n_defrole=0 then
      return 0;
    elsif n_defrole=1 then
      return 1;
    else
      begin
        select role# into n_found
        from sys.defrole$
        where user#=u
        and role#=r;
        return 3-n_defrole;
      exception
        when no_data_found then
          return n_defrole-2;
      end;
    end if;
  end;

begin
  begin
  select
  user#
  ,name
  ,type&&dbaddns
  ,' ('
    ||decode(type&&dbaddns,1,'user','role')
    ||', '||decode(&&lv2,0,'all',to_char(abs(&&lv2)))||' level(s)'
    ||decode(sign(&&lv2),-1,', reversed')
    ||')'
  into n,v_name,n_type,v_text
  from sys.user$
  where name like &&usr;
  exception
    when no_data_found then
      &&putl('No rows found.');
      return;
  end;

  &&putl(v_name||v_text);
  for r in (
    select
    level
    ,decode(sign(&&lv2),-1,grantee#,privilege#) p#
    from sys.sysauth$ sa
    where privilege# >= 0 
    and (
      &&lv2 >= 0
      or not exists (
        select 0
        from sys.user$
        where user# = sa.grantee#
        and type&&dbaddns = 1
      )
    ) 
    and (&&lv2 = 0 OR level <= abs(&&lv2))
    start with (
      (&&lv2 >= 0 and grantee# = n)
      or (&&lv2 < 0 and privilege# = n)
    )
    connect by (
      (&&lv2 >= 0 and grantee# = prior privilege#)
      or (&&lv2 < 0 and privilege# = prior grantee#)
    )
  ) loop
    v_out:=rpad('.',2*r.level)||get_role_name(r.p#);
    if n_type=1 and r.level=1 and get_def_role(n,r.p#)=1 then
      v_out:=v_out||' (default)';
    end if;
    &&putl(v_out);
  end loop;
end;
/

undef usr lvl lvl2

@setdefs

