rem Author:  Mark Lang, 1998
rem Name:    hits.sql
rem Purpose: Display primary ORACLE internal hit ratios and memory
rem Usage:   @hits <db>
rem Subject: tuning
rem Attrib:  sql dba
rem Descr:
rem Notes:   LC=library cache
rem          DC=dictionary cache
rem          BC=buffer cache
rem          SORTS=sorts to disk
rem          (include recomended values)
rem SeeAlso: @ratio
rem History:
rem          01-feb-98  Initial release


@setup
define db=""
set heading off
set serveroutput on
define l="6"

select
  'PERFORMANCE RATIOS - '||name
from v$database&&curdb
;

column sort1 noprint
column sort2 noprint
column name format a10
column value format 0.999
column opt format 0.999
column params format a40

select
  1 sort1,'LC',1-(sum(lc.reloads)/sum(lc.pins)) value,0.99 opt,
  ltrim(to_char(max(p1.value)/(1024*1024), '990'))||'M' params
from v$librarycache&&curdb lc, v$parameter&&curdb p1
where p1.name = 'shared_pool_size'
union
select
  2,'DC',(sum(dc.gets)-sum(dc.getmisses))/sum(dc.gets),0.85,
  ltrim(to_char(max(p1.value)/(1024*1024), '990'))||'M' params
from v$rowcache&&curdb dc, v$parameter&&curdb p1
where p1.name = 'shared_pool_size'
union
select
  3,'BC',1-(pr.value/(bg.value+cg.value)),0.85,
  ltrim(to_char((p1.value*p2.value)/(1024*1024), '990'))||'M ('||
  ltrim(to_char(p2.value,'99,990'))||' '||to_char(p1.value/1024)||'k blocks)'
from v$sysstat&&curdb bg, v$sysstat&&curdb cg, v$sysstat&&curdb pr,
  v$parameter&&curdb p1, v$parameter&&curdb p2
where bg.name = 'db block gets'
and cg.name = 'consistent gets'
and pr.name = 'physical reads'
and p1.name = 'db_block_size'
and p2.name = 'db_block_buffers'
union
select
  4, 'SORTS',1-(sd.value/(sm.value+sd.value)),0.95,
  ltrim(to_char(p1.value/1024, '9,990'))||'k/'||
  ltrim(to_char(p2.value/1024, '9,990'))||'k'
from v$sysstat&&curdb sm, v$sysstat&&curdb sd,
  v$parameter&&curdb p1, v$parameter&&curdb p2
where sm.name = 'sorts (memory)'
and sd.name = 'sorts (disk)'
and p1.name = 'sort_area_size'
and p2.name = 'sort_area_retained_size'
;

undef db l

@setdefs





