rem Author:  Mark Lang, 1998
rem Name:    io.sql
rem Purpose: Display Oracle i/o distribution by file, tablespace, or device
rem Usage:   @io <f|t|d>
rem Subject: tuning
rem Attrib:  sql
rem Descr:
rem Notes:   f=by file, t=by tspace, d=by device/mount point
rem SeeAlso:
rem History:
rem          01-feb-98  Initial release

@setup
define db=""
define nm="'&&2'"

VAR n_rd NUMBER
VAR n_wr NUMBER

SET TERMOUT OFF
BEGIN
  SELECT
    SUM(phyblkrd), SUM(phyblkwrt)
    INTO :n_rd, :n_wr
  FROM v$filestat&&db;
end;
/
SET TERMOUT ON

COLUMN name     FORMAT a28      HEADING "Name"
COLUMN tsize	FORMAT 9,990 	HEADING "MB"
COLUMN reads    FORMAT 999,990  HEADING "PhyRead"
COLUMN readp    FORMAT 999      HEADING "RD%" JUSTIFY RIGHT
COLUMN writs    FORMAT 999,990  HEADING "PhyWrite"
COLUMN writp    FORMAT 990      HEADING "WR%" JUSTIFY RIGHT
COLUMN total    FORMAT 999,990  HEADING "PhyTotal"
COLUMN totlp    FORMAT 990      HEADING "TL%" JUSTIFY RIGHT

DEFINE nameln=30
DEFINE devsep="GREATEST(0,INSTR(df.name,'/',-1),INSTR(df.name,':',-1),INSTR(df.name,'\',-1))"
DEFINE device="SUBSTR(df.name,1,&&devsep)"
DEFINE filenm="SUBSTR(df.name,&&devsep+1)"
define name="decode(upper('&&1'),'T',ts.name,'D',&&device,df.name)"

SELECT
  max(&&name) name,
/*
  DECODE(UPPER('&&1'),
    'TS',  MAX(ts.name),
    'DEV', MAX(&&device),
     MAX(df.name)
  ) name,
*/
  SUM(df.bytes/(1024*1024)) tsize,
  SUM(st.phyblkrd)/1000 reads,
  SUM(st.phyblkrd)/(:n_rd)*100 readp,
  SUM(st.phyblkwrt)/1000 writs,
  SUM(st.phyblkwrt)/(:n_wr)*100 writp,
  SUM(st.phyblkrd+st.phyblkwrt)/1000 total,
  SUM(st.phyblkrd+st.phyblkwrt)/(:n_rd+:n_wr)*100 totlp
FROM v$datafile&&db df, v$filestat&&db st, sys.file$&&db sf, sys.ts$&&db ts
WHERE df.file# = st.file#
AND df.file# = sf.file#
AND sf.ts# = ts.ts#
and &&name like &&nm
GROUP BY
  DECODE(UPPER('&&1'),
    'T', ts.name,
    'D', &&device,
    df.file#)
;

undef db nm

@setdefs

