rem Author:  Mark Lang, 1998
rem Name:    jc.sql
rem Purpose: Job control
rem Usage:   @jc <job#> <flags[*]> <arg|%>
rem Subject: job
rem Attrib:  pls dml
rem Descr:
rem Notes:   flags:
rem             b=break    d=remove     x=run
rem             w=what     n=next_date  i=interval
rem             p=check_privs
rem          *=Performs a COMMIT
rem SeeAlso: @jobs @submit
rem History:
rem          23-feb-98  Initial release

@setup

define job="to_number('&&1')"
define flg="upper(substr('&&2',1,1))"
define cmt="upper(substr('&&2',2,1))"
define arg="'&&3'"
define dat="to_date('&&3','DD-MON-RR HH24:MI')"

declare
  c char(1);
  d date;
  f boolean;
  j binary_integer;
begin
  c := &&flg;
  if c = 'N' then
    select &&dat
    into d
    from dual;
  end if;
  j := &&job;
  --&&putl(c);
  if    c = 'B' then
    f := (upper(&&arg) = 'Y');
    if f then
      sys.dbms_job.broken(j, true);
      &&putl('Job broken.');
    else
      sys.dbms_job.broken(j, false);
      &&putl('Job unbroken.');
    end if;
  elsif c = 'X' then
    sys.dbms_job.run(j);
    &&putl('Job executed.');
  elsif c = 'D' then
    sys.dbms_job.remove(j);
    &&putl('Job removed.');
  elsif c = 'N' then
    sys.dbms_job.next_date(j, d);
    &&putl('Job date altered.');
  elsif c = 'I' then
    sys.dbms_job.interval(j, &&arg);
    &&putl('Job interval altered.');
  elsif c = 'W' then
    sys.dbms_job.what(j, &&arg);
    &&putl('Job altered.');
  elsif c = 'P' then
    sys.dbms_job.check_privs(j);
    &&putl('Job privileges checked.');
  else
    &&putl('Invalid flag');
  end if;
  if &&cmt = '*' then
    commit;
    &&putl('Commit complete.');
  end if;
end;
/

@setdefs
