rem Author:  Mark Lang, 1998
rem Name:    pubsyn.SQL
rem Purpose: Resolve public synonyms for objects in a schema
rem Usage:   @pubsyn <%owner.name%>
rem Subject: object:synonym
rem Attrib:  sql dba
rem Descr:   This useful script will maintain PUBLIC SYNONYMS for objects
rem          in a schema
rem             If existing synonym exists which points to something 
rem             different the drop existing
rem             If synonym does not exist then create
rem             If existing synonym points to non-existant object then drop
rem             Does not create public synonyms for synonyms
rem Notes:   Currently OWNER cannot be a wildcard
rem SeeAlso: @pubs
rem History:
rem          01-feb-98  Initial release

@setup1
set heading off

spool pubsyn.tmp

select
  decode(s.synonym_name, null, '',
    '/* was '||s.table_owner||'.'||s.table_name||
      decode(s.db_link, null, '', '@'||s.db_link)||' */'||CHR(10)||
    'DROP PUBLIC SYNONYM '||object_name||';'||chr(10))||
  'CREATE PUBLIC SYNONYM '||object_name||chr(10)||
  '  FOR '||&&o1||'.'||object_name||';'
from &&ora._objects o, &&ora._synonyms s
where
  (o.owner = &&o1) and
  (o.object_name like &&n1) and
  (o.object_name = s.synonym_name(+)) and
  (o.object_type in ('TABLE', 'VIEW', 'SEQUENCE', 'PACKAGE', 'PROCEDURE',
     'FUNCTION', 'SNAPSHOT')) and
  (s.owner is null or
    (s.owner = 'PUBLIC' and
      (s.table_owner <> &&o1 or s.table_name <> o.object_name or
         s.db_link is not null)
      and
      (s.table_owner not in ('SYS', 'SYSTEM'))))
order by o.object_name
;

rem drop all other public synonyms which reference objects in user's
rem account which no longer exist.

select
  'DROP PUBLIC SYNONYM '||synonym_name||';'
from &&ora._synonyms s
where s.owner = 'PUBLIC'
and s.table_name <> 'SYS'
and s.table_owner like &&o1
and s.table_name like &&n1
and s.db_link is null
and not exists (
  select 0
  from &&ora._objects
  where owner = s.table_owner
  and object_name = s.table_name
)
order by s.synonym_name
;

spool off
set feedback on
@doit
@pubsyn.tmp
@setdefs
