rem Author:  Mark Lang, 1998
rem Name:    reftree.sql
rem Purpose: Display FK references in tree format
rem Usage:   @reftree <%owner.table%> <cnt_refs> <level>
rem Subject: object:table:constraint
rem Attrib:  pls dba n80
rem Descr:
rem Notes:   if <cnt_refs> >=0 then table must have no more than N refs
rem          if <cnt_refs> < 0 then table must have less than N refs
rem          if <level> > 0 then display N refs
rem          if <level> = 0 then display all refs
rem          if <level> < 0 then display inverse to N levels
rem SeeAlso: @refs
rem History:
rem          05-feb-98  Initial release

@setup1
@output

define cnt="decode('&&2','%',99,to_number('&&2'))"
define lvl="decode('&&3','%',0,to_number('&&3'))"

declare
  n_own number;
  n_obj number;
  n_lvl number;
  v_text varchar2(240);

  function get_obj_name(n_obj number)
  return varchar2
  is
    retval varchar2(60);
  begin
    select u.name||'.'||o.name
    into retval
    from sys.obj$ o, sys.user$ u
    where o.obj#=n_obj
    and o.owner#=u.user#;
    return retval;
  end;

  procedure get_refs(n_obj number, n_level number)
  is
  begin
    if n_level>abs(n_lvl) and abs(n_lvl)>0 then
      return;
    end if;

    if n_lvl>=0 then
      for p in (
	select *
	from sys.cdef$
	where obj#=n_obj
	and type in (2,3)       -- PK and UK
      ) loop
	for r in (
	  select *
	  from sys.cdef$
	  where robj#=p.obj#
	  and rcon#=p.con#
	) loop
	  sys.dbms_output.put_line(rpad('.',n_level*2)||get_obj_name(r.obj#));
	  get_refs(r.obj#,n_level+1);
	end loop;
      end loop;
    else
      for p in (
	select *
	from sys.cdef$
	where obj#=n_obj
	and type = 4       -- FK
      ) loop
	for r in (
	  select *
	  from sys.cdef$
	  where robj#=p.obj#
	  and rcon#=p.con#
	) loop
	  sys.dbms_output.put_line(rpad('.',n_level*2)||get_obj_name(r.obj#));
	  get_refs(r.obj#,n_level+1);
	end loop;
      end loop;
    end if;
  end; 

begin
  select &&lvl into n_lvl
  from sys.dual;

  for r in (
    select obj#
    from sys.obj$ o, sys.user$ u
    where o.owner#=u.user#
    and u.name like &&o1
    and o.name like &&n1
    and o.type=2        -- TABLE
    and (
      (&&cnt>=0 
	and &&cnt>=(
	  select count(*)
	  from sys.cdef$
	  where robj#=o.obj#
	  and type=4
      ))
      or (&&cnt<0
	and &&cnt>(
	  select count(*)
	  from sys.cdef$
	  where robj#=o.obj#
	  and type=4
      ))
    )
  ) loop
    sys.dbms_output.put_line(get_obj_name(r.obj#));
    get_refs(r.obj#,1);
  end loop;
   
end;
/

undef cnt lvl

@setdefs

