rem Author:  Mark Lang, 1998
rem Name:    rels.sql
rem Purpose: Display relationships between tables
rem Usage:   @rels <%owner.child%> <%owner.parent%>
rem Subject: object:table:constraint
rem Attrib:  pls dba
rem Descr:
rem Notes:   Relations of format
rem          (opt|man) [m:1 (opt:man)
rem          [mM1][:+]1
rem          1:1  one to one , foreign key in primary key
rem          1+1  one to one , foreign key prefix of primary key
rem          M:1  one to many, foreign key in primary key
rem          M+1  one to many, foreign key prefix of primary key
rem          m:1  one to many  (normal)
rem SeeAlso: @cons @refs
rem History:
rem          01-feb-98  Initial release

@setup2
set serveroutput on size 102400

define set1="select column_name from sys.dba_cons_columns where owner=r.owner and constraint_name = r.constraint_name"
define set2="select column_name from sys.dba_cons_columns where owner=s.owner and constraint_name = s.constraint_name"

declare
  i number;
  m number;
  n number;
  num_cols number;
  num_null number;
  rel_type number;
  result varchar2(255);
begin
  for r in (
    select 
      c.*,
      r.table_name r_table_name,
      r.constraint_type r_constraint_type
    from sys.dba_constraints c, sys.dba_constraints r
    where c.owner like &&o1
    and c.table_name like &&n1
    and c.constraint_type = 'R'
    and r.owner like &&o2
    and r.table_name like &&n2
    and c.r_owner = r.owner
    and c.r_constraint_name = r.constraint_name
    order by c.owner, c.table_name
  ) loop
--    &&putl(r.table_name||'.'||r.constraint_name);

    result := rpad(r.owner||'.'||r.table_name, 30);

    select count(*), sum(decode(t.nullable, 'Y', 1, 0))
    into num_cols, num_null
    from sys.dba_cons_columns c, sys.dba_tab_columns t
    where c.owner = r.owner and c.constraint_name = r.constraint_name
    and c.owner = t.owner and c.table_name = t.table_name
    and c.column_name = t.column_name;

    for s in (
      select *
      from sys.dba_constraints c
      where c.owner = r.owner
      and c.table_name = r.table_name
      and c.constraint_type in ('P','U')
    ) loop
      rel_type := 0;

      select count(*)		-- is part of UNIQUE or PRIMARY key
      into n
      from dual
      where exists (
        &&set1
        minus
        &&set2
      );

      if n = 0 then
        if s.constraint_type = 'P' then
          rel_type := 2;
        else
          rel_type := 1;
        end if;

        select count(*)		-- is one-to-one
        into m
        from dual
        where exists (
          &&set2
          minus
          &&set1
        );

        if m = 0 then
          if s.constraint_type = 'P' then
            rel_type := 4;
          else
            rel_type := 3;
          end if;
          exit;
        end if;
      end if;

    end loop;

    if num_null > 0 then
      result := result||' (opt)';
    else
      result := result||' (man)';
    end if;

    if rel_type = 4 then result := result||' 1+1';
    elsif rel_type = 3 then result := result||' 1:1';
    elsif rel_type = 2 then result := result||' M+1';
    elsif rel_type = 1 then result := result||' M:1';
    else result := result||' m:1';
    end if;

    result := result||' (opt)';
    result := result||' '||rpad(r.r_owner||'.'||r.r_table_name, 30);

    &&putl(result);
  end loop;
end;
/

undef set1 set2

@setdefs




