rem Name:    restidxs.sql
rem Purpose: Restore indexes defined in SAVE_INDEXES table
rem Usage:   @restidxs <%>
rem Subject: misc
rem Attrib:  sql dyn ddl
rem Descr:
rem Notes:
rem SeeAlso: @saveidxs
rem History:
rem          23-mar-98  Initial release


@setup

declare

function exec(stmt varchar2)
return number is
  i number;
  retval number;
begin
  i := sys.dbms_sql.open_cursor;
  sys.dbms_sql.parse(i, stmt, sys.dbms_sql.v7);
  retval := sys.dbms_sql.execute(i);
  sys.dbms_sql.close_cursor(i);
  return retval;
end exec;


procedure doit(stmt varchar2)
is
  retval number;
begin
  retval:=exec(stmt);
end;

  
procedure restore_indexes(v_owner varchar2, v_table varchar2 default '%',
  v_index varchar2 default '%',
  flags varchar2 default '',
  save_tab varchar2 default 'SAVED_INDEXES'
)
is
  c1 integer;
  c2 integer;
  cmd varchar2(2000);
  name varchar2(30); 
  res varchar2(2000);
  ok integer;
  t1 date;
  t2 date;
begin
  cmd := 'select index_name,text from '||save_tab||' '||
         'where owner like '''||upper(v_owner)||''' '||
         'and table_name like '''||upper(v_table)||''' '||
         'and index_name like '''||upper(v_index)||''' '||
         'order by owner,table_name,index_name ';

  c1 := sys.dbms_sql.open_cursor;
  sys.dbms_sql.parse(c1, cmd, sys.dbms_sql.v7);
  sys.dbms_sql.define_column(c1, 1, name, 30);
  sys.dbms_sql.define_column(c1, 2, res, 2000);
 
  ok := sys.dbms_sql.execute(c1);
 
  loop
    if sys.dbms_sql.fetch_rows(c1) = 0 then
      exit;
    end if;
    sys.dbms_sql.column_value(c1, 1, name);
    sys.dbms_sql.column_value(c1, 2, res);
    begin
      if instr(upper(flags),'U') > 0 then
        res := res||' unrecoverable';
      end if;
      t1 := sysdate;
      doit(res);
      t2 := sysdate;
      sys.dbms_output.put_line('Restored index '||name||
        ' ('||ltrim(to_char((t2-t1)*24*60,'99,990.99'))||').');
    exception 
      when others then 
        sys.dbms_output.put_line('Failed to restore index '||name||'...');
        --sys.dbms_output.put_line(substr(res,1,240));
        sys.dbms_output.put_line(substr(sqlerrm,1,240));
    end;
    --sys.dbms_output.put_line(substr(res,1,240));
  end loop;
  sys.dbms_sql.close_cursor(c1);
 
exception
  when others then
    if sys.dbms_sql.is_open(c1) then
      sys.dbms_sql.close_cursor(c1);
    end if;
    raise;
end restore_indexes;

begin
restore_indexes(user,'&&1','&&2','&&3');
end;
/

@setdefs
