rem Author:  Mark Lang, 1998
rem Name:    space8.sql
rem Purpose: Display unused space within segment extents
rem Usage:   @space8 <%owner.name%> <%type%> <%tablespace%>
rem Subject: space
rem Attrib:  sql v80
rem Descr:
rem Notes:   enh: merge with @space in future release
rem          fix: got error for object type in system account
rem SeeAlso: @space
rem History:
rem          01-feb-98 Initial release

@setup1
set serveroutput on size 102400
define ty="upper('&&2')"
define ts="upper('&&3')"
define lo="0.750"
define hi="0.950"
define pr="1-(unused_blocks/total_blocks)"

declare
  total_blocks number;
  total_bytes number;
  unused_blocks number;
  unused_bytes number;
  last_used_extent_file_id number;
  last_used_extent_block_id number;
  last_used_block number;
  free_blks number;
  c char(1);

  function fmt(n number) return varchar2 is
  begin
    if n < 1024 then
      return to_char(n,'9,999')||' ';
    elsif n < 1024*1024 then
      return to_char(n/1024,'9,999')||'K';
    elsif n < 1024*1024*1024 then
      return to_char(n/(1024*1024),'9,999')||'M';
    else
      return to_char(n/(1024*1024*1024),'9,999')||'G';
    end if;
  end;

begin

  &&putl(rpad('SEGMENT',25)||' '||
    rpad('TYPE',12)||'  '||
    lpad('TOTAL',6)||'  '||
    lpad('USED', 6)||'  '||
    lpad('AVAIL',6)||'  '||
    rpad('CAPACITY',12));

  for r in (
    select &&segown owner,segment_name,segment_type,partition_name
    from &&oru._segments s
    where &&segown like &&o1 
    and segment_name like &&n1
    and segment_type like &&ty
    and tablespace_name like &&ts
    order by &&segown, segment_name, segment_type
  ) loop
   
    sys.dbms_space.unused_space(
      r.owner, r.segment_name, r.segment_type, 
      total_blocks,
      total_bytes,
      unused_blocks,
      unused_bytes,
      last_used_extent_file_id,
      last_used_extent_block_id,
      last_used_block,
      r.partition_name 
    );

    if &&pr > &&hi then c:='-';
    elsif &&pr < &&lo then c:='+';
    else c:='';
    end if;

    &&putl(rpad(r.owner||'.'||nvl(r.partition_name,r.segment_name),25)||' '||
      rpad(r.segment_type,12)||' '||
      fmt(total_bytes)||' '||
      fmt(total_bytes-unused_bytes)||' '||
      fmt(unused_bytes)||' '|| 
      to_char(&&pr,'0.999')||' '||c
    );

  end loop;
end;
/

undef ty ts lo hi pr

@setdefs

