rem Author:  Mark Lang, 1998
rem Name:    stats.sql
rem Purpose: Shows session/system statistics
rem Usage:   @stats <%user%|0|aggr={sum|min|max|avg|std|var|dis}> <%stat%> <minval|%>
rem Subject: instance
rem Attrib:  sql dba
rem Descr:
rem Notes:   <user>=user stats
rem          0=system stats
rem          <aggr>=aggregate stats for all users
rem SeeAlso:
rem History:
rem          01-feb-98 Initial release


@setup
define db=""
define usr="upper('&&1')"
define stn="upper('&&2')"
define min="decode('&&3','%',0,to_number('&&3'))"

column stat#    format 9990           heading "STAT#"
column sid      format 9990           heading "SID"
column name     format a15            heading "NAME"
column statname format a35            heading "STATISTIC"
column value    format 99,999,999,990 heading "VALUE"

rem User stats

select
  s.statistic# stat#,
  s.sid sid,
  u.username name,
  n.name statname,
  s.value value
from v$sesstat&&db s, v$statname&&db n, v$session&&db u
where &&usr not in ('0', 'SUM', 'MIN', 'MAX', 'AVG', 'STD','VAR')
and (
  u.username like &&usr
  or (&&usr = '?' and u.audsid = userenv('sessionid'))
)
and s.statistic# = n.statistic#
and s.sid = u.sid
and upper(n.name) like &&stn
and value >= &&min
order by decode(&&min, 0, u.username, 'X'), n.name, s.value
;

rem System stats

select
  s.statistic# stat#,
  0 sid,
  d.name,
  s.name statname,
  s.value value
from v$sysstat&&db s, v$database&&db d
where &&usr = '0'
and upper(s.name) like &&stn
order by s.name
;

rem Current stats

select
  max(s.statistic#) stat#,
  0 sid,
  max(d.name)||' '||&&usr||' of '||ltrim(to_char(count(*))) name,
  max(n.name) statname,
  decode(&&usr,
    'MIN', min(s.value),
    'MAX', max(s.value),
    'AVG', avg(s.value),
    'STD', stddev(s.value),
    'VAR', variance(s.value),
    sum(s.value)
  ) value
from v$sesstat&&db s, v$statname&&db n, v$database&&db d
where &&usr IN ('SUM', 'MIN', 'MAX', 'AVG', 'STD', 'VAR')
and s.statistic# = n.statistic#
and upper(n.name) like &&stn
group by s.statistic#,n.name
order by n.name
;

rem Distinct Values

select
  max(s.statistic#) stat#,
  0 sid,
  max(d.name)||' DISTINCT '||ltrim(to_char(count(*))) name,
  n.name statname,
  s.value value
from v$sesstat&&db s, v$statname&&db n, v$session&&db u, v$database&&db d
where &&usr in ('DIS')
and s.statistic# = n.statistic#
and s.sid = u.sid
and upper(n.name) like &&stn
group by n.name, s.value
;

undef db usr stn min

@setdefs
