rem Author:  Mark Lang, 1998
rem Name:    syncgrnt.sql
rem Purpose: Synchronize grants between databases
rem Usage:   @syncgrnt <database> <%grantee%> <usertype> <priv> <grant_rev>
rem Subject: security
rem Attrib:  sql
rem Descr:
rem Notes:   <usertype>  = <UR|%>
rem          <priv> = <SRT|%>
rem          <grant_rev> = <GR|%>
rem          - Does not effect SYS, SYSTEM, or system roles
rem          - Does not create or drop USERS or ROLES
rem          - Only grants or revoke is USER+ROLE+OBJECT exists in BOTH dbs
rem          - Can only sync objects granted for current user
rem SeeAlso: @syncpwd
rem History:

@setup
set heading off

define db="@&&1"
define usr="upper('&&2')"
define prv="replace(upper('&&3'),'%','SRTC')"
define arg="replace(upper('&&4'),'%','G')"

spool syncgrnt.tmp

rem System Privileges

select
  'revoke '||p.privilege||' from '||grantee||';'
from sys.dba_sys_privs p
where instr(&&arg, 'R') > 0
and instr(&&prv,'S') > 0
and grantee like &&usr
and grantee not in (&&sysusers, &&sysroles)
and exists (                    -- role exists
  select 0
  from sys.dba_roles&&db
  where role = p.grantee
)
and not exists (                -- grant does not
  select 0
  from sys.dba_sys_privs&&db
  where grantee = p.grantee
  and privilege = p.privilege
  and admin_option = p.admin_option
);

select
  'grant '||privilege||' to '||grantee||
  decode(admin_option, 'YES', &&cr||'with admin option', '')||';'
from sys.dba_sys_privs&&db p
where instr(&&arg, 'G') > 0
and instr(&&prv, 'S') > 0
and grantee like &&usr
and grantee not in (&&sysusers, &&sysroles)
and exists (                    -- local role exists
  select 0
  from sys.dba_roles
  where role = p.grantee
)
and not exists (                -- privilege does not
  select 0
  from sys.dba_sys_privs
  where grantee = p.grantee
  and privilege = p.privilege
  and admin_option = p.admin_option
);

rem Role Privileges

select
  'revoke '||granted_role||' from '||grantee||';'
from sys.dba_role_privs p
where instr(&&arg, 'R') > 0
and instr(&&prv,'R') > 0
and grantee like &&usr
and grantee not in (&&sysusers, &&sysroles)
and exists (                    -- parent role exists
  select 0
  from sys.dba_roles&&db
  where role = p.grantee
)
and exists (                    -- child role exists
  select 0
  from sys.dba_roles&&db
  where role = p.granted_role
)
and not exists (                -- grant does not exist
  select 0
  from sys.dba_role_privs&&db
  where grantee = p.grantee
  and granted_role = p.granted_role
  and admin_option = p.admin_option
);

select
  'grant '||granted_role||' to '||grantee||
  decode(admin_option, 'yes', &&cr||'with admin option', '')||';'
from sys.dba_role_privs&&db p
where instr(&&arg, 'G') > 0
and instr(&&prv,'R') > 0
and grantee like &&usr
and grantee not in (&&sysusers, &&sysroles)
and exists (                    -- parent role exists
  select 0
  from sys.dba_roles
  where role = p.grantee
)
and exists (                    -- child role exists
  select 0
  from sys.dba_roles
  where role = p.granted_role
)
and not exists (                -- grant does not exist
  select 0
  from sys.dba_role_privs
  where grantee = p.grantee
  and granted_role = p.granted_role
  and admin_option = p.admin_option
);

rem Object Privileges

select
  'revoke '||privilege||' on '||owner||'.'||table_name||&&cr||
  'from '||grantee||';'
from sys.dba_tab_privs p
where instr(&&arg, 'R') > 0
and instr(&&prv,'T') > 0
and grantee like &&usr
and grantee not in (&&sysusers, &&sysroles)
and owner = user and grantor = user
and exists (                    -- role exists
  select 0
  from sys.dba_roles&&db
  where role = p.grantee
)
and exists (                    -- object exists
  select 0
  from sys.dba_objects&&db
  where owner = p.owner
  and object_name = p.table_name
)
and not exists (                -- grant does not exist
  select 0
  from sys.dba_tab_privs&&db
  where grantee = p.grantee
  and owner = p.owner
  and table_name = p.table_name
  and grantor = p.grantor
  and privilege = p.privilege
  and grantable = p.grantable
);

select
  'grant '||privilege||' on '||owner||'.'||table_name||&&cr||
  'to '||grantee||decode(grantable,'YES',' with grant option','')||';'
from sys.dba_tab_privs&&db p
where instr(&&arg, 'G') > 0
and instr(&&prv,'T') > 0
and grantee like &&usr
and grantee not in (&&sysusers, &&sysroles)
and owner = user and grantor = user
and exists (                    -- role exists
  select 0
  from sys.dba_roles
  where role = p.grantee
)
and exists (                    -- object exists
  select 0
  from sys.dba_objects
  where owner = p.owner
  and object_name = p.table_name
)
and not exists (                -- grant does not exist
  select 0
  from sys.dba_tab_privs
  where grantee = p.grantee
  and owner = p.owner
  and table_name = p.table_name
  and grantor = p.grantor
  and privilege = p.privilege
  and grantable = p.grantable
);

spool off

@doit
set feedback on
@syncgrnt.tmp
@setdefs



