rem Author:  Mark Lang, 1998
rem Name:    tabstat.sql
rem Purpose: List table statistics
rem Usage:   @tabstat <%owner.table%> <%ts%>
rem Subject: object:table
rem Attrib:  sql
rem Descr:
rem Notes:   Theses columns in SYS.DBA_TABLES are calculated by ANALYZE
rem		NUM_ROWS	number of rows 
rem		BLOCKS		blocks below high-water mark
rem		EMPTY_BLOCKS	blocks above high-water mark
rem		AVG_SPACE	average free space in blocks below high-water
rem		CHAIN_CNT	number of chained or migrated rows
rem SeeAlso:
rem History:
rem          01-feb-98  Initial release


@setup1
define ts="upper('&&2')"

column tname format a25 heading "TABLE_NAME"
column tablespace_name format a15 heading "TABLESPACE"
column num_rows format 999,999,990
column empty_pct format a5 heading "EMPTY"
column avg_space format 9990 heading "AVGSP"
column chain_pct format 0.999 heading "CHAIN"
column avg_row_len format 9990 heading "AVGRL"

select
  owner||'.'||table_name||' '||
  decode(cluster_name, NULL, '', &&cr||'(in '||
    cluster_name||')') tname,
  tablespace_name,  
  num_rows,
  ltrim(decode(blocks,null,'',0,to_char(0,'0.999'),
    to_char(empty_blocks/(blocks+empty_blocks), '0.999'))) empty_pct,
  avg_space,
  ltrim(decode(num_rows,null,'',0,to_char(0,'0.999'),
    to_char(chain_cnt/num_rows, '0.999'))) chain_pct,
  avg_row_len
from &&ora._tables 
where owner like &&o1
and table_name like &&n1
and (tablespace_name like &&ts 
  or tablespace_name is null) -- O8
order by owner, table_name
;

undef ts

@setdefs
