rem Author:  Mark Lang, 1998
rem Name:    whatis.sql
rem Purpose: Resolve reference to specified name from current account
rem Usage:   @whatis <name>
rem Subject: object
rem Attrib:  sql
rem Descr:
rem Notes:   resolution stops if object is in another database
rem          Tells you what object <name> actually refers to
rem SeeAlso:
rem History:
rem          01-feb-98  Initial release

@setup
set heading off

var result varchar2(80)

declare
  a       varchar2(30);
  b       varchar2(30);
  c       varchar2(30);
  dblink  varchar2(128);
  nextpos number;

  function dig_it ( v_owner varchar2, v_name varchar2 )
  return varchar2 is
    r &&ora._objects%rowtype;
    s &&ora._synonyms%rowtype;
    p varchar2(30);
  begin

    if v_owner is null or v_name is null then
      return '';
    end if;

    begin
      select * into s
      from &&ora._synonyms
      where owner = v_owner and synonym_name = v_name;
      exception when no_data_found then
        begin
          select * into r
          from &&ora._objects
          where owner = v_owner
          and object_name = v_name
          and rownum = 1;
          exception when no_data_found then
            return null;
        end;
        return r.owner||'.'||r.object_name||' ('||r.object_type||')';
    end;

    if s.db_link is not null then
      return s.owner||'.'||s.synonym_name||' => '||nvl(s.table_owner, '?')||
        '.'||s.table_name||'@'||s.db_link||' (???)';
    else
      return v_owner||'.'||v_name||' => '||dig_it(s.table_owner, s.table_name);
    end if;
  end;

begin
  dbms_utility.name_tokenize(upper('&&1'), a, b, c, dblink, nextpos);

  if b is null then
    b := a;
    a := user;
  end if;

  if dblink is null then
    :result := dig_it(a, b);

    if :result is null and a = user then
      :result := dig_it('PUBLIC', b);
    end if;

    if :result is null then
      :result := 'Item does not exist.';
    end if;
  else
    :result := 'Cannot resolve a remote object.';
  end if;
end;
/

PRINT result

@setdefs
